# Generated by Django 2.0.7 on 2018-09-30 01:17

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dynamic_business_rules', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='businessruleset',
            name='signal_kwargs',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, help_text="kwargs to pass to signal registration ({'sender': 'Model' would be a common pattern here})"),
        ),
        migrations.AlterField(
            model_name='condition',
            name='property_kwargs',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, help_text='kwargs to pass to the method that gets the left hand side of our comparison'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='property_method',
            field=models.CharField(blank=True, max_length=140, null=True),
        ),
        migrations.AlterField(
            model_name='condition',
            name='value',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='conditionset',
            name='combination_rule',
            field=models.CharField(choices=[('any', 'any'), ('all', 'all')], default='all', max_length=10),
        ),
        migrations.AlterField(
            model_name='historicalbusinessruleset',
            name='signal_kwargs',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, help_text="kwargs to pass to signal registration ({'sender': 'Model' would be a common pattern here})"),
        ),
        migrations.AlterField(
            model_name='historicalcondition',
            name='property_kwargs',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, help_text='kwargs to pass to the method that gets the left hand side of our comparison'),
        ),
        migrations.AlterField(
            model_name='historicalcondition',
            name='property_method',
            field=models.CharField(blank=True, max_length=140, null=True),
        ),
        migrations.AlterField(
            model_name='historicalcondition',
            name='value',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='historicalconditionset',
            name='combination_rule',
            field=models.CharField(choices=[('any', 'any'), ('all', 'all')], default='all', max_length=10),
        ),
    ]
